#ifndef REGISTRY_H_INC
#define REGISTRY_H_INC

class RegistryClass
{
public:
   RegistryClass(const char *aKey)
   {
      if(RegOpenKey(HKEY_CURRENT_USER, aKey, &KeyHandle) != ERROR_SUCCESS)
      {
         if(RegCreateKeyEx(HKEY_CURRENT_USER, aKey, 0, 0, REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, 0, &KeyHandle, 0) != ERROR_SUCCESS)
            KeyHandle = 0;
      }
   }
   ~RegistryClass()
   {
      if(KeyHandle)
         RegCloseKey(KeyHandle);
   }
   int GetInt(const char *aKey, int aDefaultValue = 0)
   {
      if(!KeyHandle)
        return aDefaultValue;

      DWORD Type = REG_DWORD, Size = sizeof(DWORD);
      int Value;
      if(RegQueryValueEx(KeyHandle, aKey, 0, &Type, (BYTE *)&Value, &Size) != ERROR_SUCCESS)
         return aDefaultValue;

      return Value;
   }

   void WriteInt(const char *aKey, int aValue)
   {
      if(!KeyHandle)
         return;
      RegSetValueEx(KeyHandle, aKey, 0, REG_DWORD, (BYTE *)&aValue, sizeof(aValue));
   }

   char *GetString(const char *aKey, char *aBuffer, int aMaxSize)
   {
      if(!KeyHandle)
      {
         *aBuffer = 0;
         return aBuffer;
      }
      DWORD Type = REG_SZ, Size = aMaxSize;
      if(RegQueryValueEx(KeyHandle, aKey, 0, &Type, (BYTE *)aBuffer, &Size) != ERROR_SUCCESS)
         *aBuffer = 0;

      return aBuffer;
   }

   void WriteString(const char *aKey, const char *aBuffer)
   {
      if(!KeyHandle)
         return;
      RegSetValueEx(KeyHandle, aKey, 0, REG_SZ, (BYTE *)aBuffer, strlen(aBuffer));
   }

private:
   HKEY KeyHandle;
};

#endif
